#!/bin/sh

LC_NUMERIC=en_US.UTF-8
export LC_NUMERIC

INPUTFILE=lab7.tr

if [ ! -f "$INPUTFILE" ]; then
	echo "ERROR: Can't find input file $INPUTFILE"
	exit 1
elif [ ! -s "$INPUTFILE" ]; then
	echo "ERROR: Empty input file $INPUTFILE"
	exit 1
fi

cat $INPUTFILE | gawk '
	BEGIN { a1 = a5 = to_write = 0; a2 = 0.0; a3 = a4 = ""; sign=-1; }
	{
		if (($1 == "-" || $1 == "e") && $5 == "cbr") {
			if ($1 == "-") { sign = "1-"; }
			if ($1 == "e") { sign = "4e"; }
			printf("%d %f %s %s %d\n", 1000 + $12, $2, sign, $5, $6);
		}
		if ($1 == "r" && $5 == "cbr") {
			if ($1 == "r") { sign = "3r"; }
			a1 = 1000 + $12;
			a2 = $2;
			a3 = sign;
			a4 = $5;
			a5 = $6;
			to_write = 1;
		} else {
			if ($1 == "+" && $5 == "HDLC" && $20 == "RR]" ) {
				if (to_write == 1) {
					printf("%d %f %s %s %d %d\n", a1, a2, a3, a4, a5, 1);
					to_write = 0;
				}
			} else {
				if (to_write == 1) {
					printf("%d %f %s %s %d %d\n", a1, a2, a3, a4, a5, 0);
					to_write = 0;
				}
			}
		}
	}'| sort | gawk '
	BEGIN {
		cur_pack=-1;
		retransmit = unique_transmit = errors = received = found = bytes = 0;
		delay = delay_sample = start_time = end_time=0.0;
	}
	{
		if (($3 == "3r" || $3 == "1-") && $4 == "cbr") {
			if ($2 > end_time) { end_time = $2; }
			if ($2 < start_time || start_time == 0) { start_time = $2; }
			if ($1 == cur_pack) {
				if ($3 == "1-") { retransmit++; }
				if (found == 0) {
					if ($3 == "3r" && $6 == 1) {
						delay += $2 - delay_sample;
						delay_sample = 0;
						received++;
						found = 1;
						bytes += $5;
					}
				}
			} else {
				if ($3 == "1-") {
					delay_sample = $2;
					unique_transmit++;
				}
				found = 0;
				cur_pack = $1;
			}
		}
		if ($3 == "4e" && $4 == "cbr") {
			errors++;
		}
	}
	END{
		print "Unique packets transmitted	:", unique_transmit;
		print "Received packets		:", received;
		print "Mean number of transm/packet	:", (retransmit/unique_transmit)+1;
		print "Mean number of errors		:", errors/(unique_transmit+retransmit);
#		print "Mean number of errors per unique packet	:", errors/(unique_transmit);
		print "Mean delay (secs)		:", delay/received;
		print "Throughput (Mbps)		:", (1/1000000)*(8*bytes)/(end_time-start_time);
	}'

